/********************************** (C) COPYRIGHT *******************************
 * File Name          : main.c
 * Author             : WCH
 * Version            : V1.1.0
 * Date               : 2024/08/01
 * Description        : Main program body.
 *******************************************************************************/
#include "debug.h"
#include "CH392INC.h"
#include "CH392.H"
#include "dns.h"

#define Query392Interrupt() GPIO_ReadInputDataBit(GPIOA, GPIO_Pin_1)

#if (CH392_OP_INTERFACE_MODE == CH392_UART_MODE) /* SEL = 1 */
#include "CH392UART.h"
#elif (CH392_OP_INTERFACE_MODE == CH392_SPI_MODE) /* SEL = 0 */
#include "CH392SPI_HW.h"
#else
#error "Please Select Correct Communication Interface "
#endif

/**********************************************************************************/
/* Include command file */
#include "CH392CMD.h"

#define MAX_URL_SIZE 128

UINT8 url_dn1[MAX_URL_SIZE] = "www.baidu.com";
UINT8 url_dn2[MAX_URL_SIZE] = "www.google.com";
UINT8 url_dn3[MAX_URL_SIZE] = "www.wch.cn";
extern UINT8 status;
/* Common variable definition */
UINT8 MyBuffer[4096];       /* data buffer */
struct _SOCK_INF SockInf;   /* Save the socket information */
struct _CH392_SYS CH392Inf; /* Save the CH392 information */

UINT8 DNS_SERVER_IP[4]; /* DNS server: Change the value based on the actual DNS server */

const UINT16 Socket0SourPort = 1000; /* Socket 0 sour port */

/**********************************************************************************
 * Function Name  : mStopIfError
 * Description    : Display error code, and stop
 * Input          : iError
 * Output         : None
 * Return         : None
 **********************************************************************************/
void mStopIfError(UINT8 iError)
{
    if (iError == CMD_ERR_SUCCESS)
        return;                                /* Success */
    printf("Error: %02X\r\n", (UINT16)iError); /* printf error */
    while (1)
    {
        Delay_Ms(200);
        Delay_Ms(200);
    }
}

/**********************************************************************************
 * Function Name  : CH392SocketInterrupt
 * Description    : CH392 socket interrupt, called in global interrupt
 * Input          : sockindex
 * Output         : None
 * Return         : None
 **********************************************************************************/
void CH392SocketInterrupt(UINT8 sockindex)
{
    UINT8 sock_int_socket;
    UINT16 len;
    u16 i;

    sock_int_socket = CH392CMDGetSocketInt(sockindex); /* Gets the socket interrupt status */
    if (sock_int_socket & SINT_STAT_SENBUF_FREE)       /* The send buffer is free and can continue writing data to be sent */
    {
    }
    if (sock_int_socket & SINT_STAT_SEND_OK) /* Send completion interrupt */
    {
    }
    if (sock_int_socket & SINT_STAT_RECV) /* Receive interruption */
    {
        len = CH392CMDGetRecvLength(sockindex); /* Gets the length of data in the current buffer */
        printf("receive len = %d\r\n", len);
        if (status == 3 && len != 0)
        {
            CH392CMDGetRecvData(sockindex, len, dns_buf);
            status = 4;
            i = CH392CMDCloseSocket(sockindex);
            mStopIfError(i);
            printf("CloseSocket0\r\n");
        }
    }
    if (sock_int_socket & SINT_STAT_CONNECT) /* The connection is interrupted, valid only in TCP mode */
    {
        printf("SINT_STAT_CONNECT\r\n");
    }
    if (sock_int_socket & SINT_STAT_DISCONNECT) /* Disconnect interrupt, valid only in TCP mode */
    {
        printf("SINT_STAT_DISCONNECT \r\n");
    }
    if (sock_int_socket & SINT_STAT_TIM_OUT) /* Timeout interrupt, valid only in TCP mode */
    {
        printf("SINT_STAT_TIM_OUT\r\n");
    }
}

/**********************************************************************************
 * Function Name  : CH392GlobalInterrupt
 * Description    : CH392 Global interrupt function
 * Input          : None
 * Output         : None
 * Return         : None
 **********************************************************************************/
void CH392GlobalInterrupt(void)
{
    UINT16 init_status;
    UINT8 buf[10];

#if CHIP == CH392T
    init_status = CH392CMDGetGlobIntStatus_392T();
#elif CHIP == CH392F
    init_status = CH392CMDGetGlobIntStatus_392F();
#endif

    if (init_status & GINT_STAT_UNREACH) /* Unreachable interrupt,to read unreachable information */
    {
        CH392CMDGetUnreachIPPT(buf);
    }
    if (init_status & GINT_STAT_IP_CONFLI) /* IP address conflict interrupted,advice to change the IP address of the CH392 again and initialize the CH392 */
    {
    }
    if (init_status & GINT_STAT_PHY_CHANGE) /* PHY change interrupt */
    {
        printf("Init status : GINT_STAT_PHY_CHANGE\r\n");
    }
    if (init_status & GINT_STAT_DHCP)
    {
        UINT8 i;

        printf("Init status : DHCP ");
        i = CH392CMDGetDHCPStatus();
        printf("%d\r\n", i);
        if (i == 0)
        {
            UINT8 arr[20];
            CH392CMDGetIPInf(arr);
            if (memcmp(CH392Inf.IPAddr, arr, 4)) // If the IP address of CH392 changes
            {
                memcpy(CH392Inf.IPAddr, arr, 4);
                memcpy(CH392Inf.GWIPAddr, &arr[4], 4);
                memcpy(CH392Inf.MASKAddr, &arr[8], 4);

                printf("IP   :  %3d . %3d . %3d . %3d\r\n", CH392Inf.IPAddr[0], CH392Inf.IPAddr[1], CH392Inf.IPAddr[2], CH392Inf.IPAddr[3]);
                printf("GW   :  %3d . %3d . %3d . %3d\r\n", CH392Inf.GWIPAddr[0], CH392Inf.GWIPAddr[1], CH392Inf.GWIPAddr[2], CH392Inf.GWIPAddr[3]);
                printf("MASK :  %3d . %3d . %3d . %3d\r\n", CH392Inf.MASKAddr[0], CH392Inf.MASKAddr[1], CH392Inf.MASKAddr[2], CH392Inf.MASKAddr[3]);
                printf("DNS1 :  %3d . %3d . %3d . %3d\r\n", arr[12], arr[13], arr[14], arr[15]);
                printf("DNS2 :  %3d . %3d . %3d . %3d\r\n", arr[16], arr[17], arr[18], arr[19]);

                memcpy(DNS_SERVER_IP, &arr[12], 4);
                status = 1;
                printf("status = 1\r\n");
            }
        }
    }
    if (init_status & GINT_STAT_SOCK0)
    {
        CH392SocketInterrupt(0); /* socket 0 interrupt */
    }
    if (init_status & GINT_STAT_SOCK1)
    {
        CH392SocketInterrupt(1); /* socket 1 interrupt */
    }
    if (init_status & GINT_STAT_SOCK2)
    {
        CH392SocketInterrupt(2); /* socket 2 interrupt */
    }
    if (init_status & GINT_STAT_SOCK3)
    {
        CH392SocketInterrupt(3); /* socket 3 interrupt */
    }
#if CHIP == CH392T /* The 392T uses eight sockets by default and a maximum of 32 sockets. For details, see the 392 manual */
    if (init_status & GINT_STAT_SOCK4)
    {
        CH392SocketInterrupt(4); /* socket 4 interrupt */
    }
    if (init_status & GINT_STAT_SOCK5)
    {
        CH392SocketInterrupt(5); /* socket 5 interrupt */
    }
    if (init_status & GINT_STAT_SOCK6)
    {
        CH392SocketInterrupt(6); /* socket 6 interrupt */
    }
    if (init_status & GINT_STAT_SOCK7)
    {
        CH392SocketInterrupt(7); /* socket 7 interrupt */
    }
#endif
}

/**********************************************************************************
 * Function Name  : CH392Init
 * Description    : Configure IP, GWIP, and MAC for the CH392, and initialize the CH392
 * Input          : None
 * Output         : None
 * Return         : Function execution result
 **********************************************************************************/
UINT8 CH392Init(void)
{
    UINT8 i;

    i = CH392CMDCheckExist(0x65);
    if (i != 0x9a)
        return CH392_ERR_UNKNOW; /* Test cmd , return 0XFA if it fails*/
                                 /* The value 0XFA is usually a hardware error or incorrect read/write timing */
    i = CH392CMDInitCH392();     /* initialize the CH392 */
    return i;
}
/**********************************************************************************
 * Function Name  : Gpio_Init
 * Description    : initialize GPIO
 * Input          : None
 * Output         : None
 * Return         : None
 **********************************************************************************/
void Gpio_Init(void)
{
    GPIO_InitTypeDef GPIO_InitStructure;
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE);

    // 392Interrupt
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    // 392Rset
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_4;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_Init(GPIOA, &GPIO_InitStructure);
    GPIO_SetBits(GPIOA, GPIO_Pin_4);
}

/*********************************************************************
 * @fn      main
 *
 * @brief   Main program.
 *
 * @return  none
 */
int main(void)
{
    UINT8 i, ip[4];
    Delay_Init();
    USART_Printf_Init(115200);
#if (CH392_OP_INTERFACE_MODE == CH392_UART_MODE) /* SEL = 1 */
    CH392_UARTPort_Init();
#elif (CH392_OP_INTERFACE_MODE == CH392_SPI_MODE) /* SEL = 0 */
    CH392_SPIPort_Init();
#endif

    Gpio_Init();
    CH392Reset();
    Delay_Ms(100);
    printf("CH392EVT Test Demo\r\n");
    i = CH392CMDGetVer();
    printf("CH392VER : %2x\r\n", (UINT16)i);
    i = CH392Init(); /* Initialize the CH392 */
    mStopIfError(i);
    while (1)
    {                                              /* Wait for the CH392 Connect Ethernet*/
        if (CH392CMDGetPHYStatus() == PHY_DISCONN) /* Example Query whether the CH392 is connected */
        {
            Delay_Ms(200); /* If no, wait for 200MS and query again */
        }
        else
        {
            printf("CH392 Connect Ethernet\r\n"); /*When the CH392 is connected to the Ethernet, an interruption occurs */
            break;
        }
    }

    i = CH392CMDDHCPEnable(1);
    mStopIfError(i);
    while (1)
    {
        if (Query392Interrupt() == 0)
        {
            CH392GlobalInterrupt();
        }
        i = DnsQuery(0, url_dn1, ip); /* inquire"www.baidu.com" */
        if (i)
        {
            printf("Domain name: %s \n", url_dn1);
            printf("HTTPs_IP= %d.%d.%d.%d\n\n", ip[0], ip[1], ip[2], ip[3]);
            break;
        }
    }
}
